unit ReportField;

{
Author:       Juergen Ruemmler
EMail:        juergen@ruemmler-net.de
Version:      1.1. from 12.11.1999
Legal issues: Copyright (C) 1999 by Juergen Ruemmler
              Brockdorffstr. 27a, 22149 Hamburg, Germany

              TReportField ist Freeware.

Description:

   TReportField provide different Borderstyles and can build
   each character in a separate frame. Additional you can get
   and put the Styleproperties of TReportField to save on disk.

   For the different ways please have a lock at the demo.


History:

  Version    Date         Comment
  -------    ---------    -------
  1.0        11.11.1999   Initialversion
  1.1        12.11.1999   Position and Size added to TReportFieldVar,
                          NotifyEvent OnStyleChange added

Additional:
  Ill be happy, if you write me an email, if you like this component.
  If you use this component to create an new one, please sent it me and
  Im happy too.

  Good luck and have a nice day!
}

interface

Uses StdCtrls, Classes, Controls, Windows, Graphics, Dialogs;

Type
  TCrossOut = (coNone, coSimple, coDouble);
  TDividerHeight = (dhNone, dhHalf, dhFull);

  TBorder = class(TPersistent)
  private
    FTop,
    FLeft,
    FRight,
    FBottom: Boolean;
    FOnChange: TNotifyEvent;
    procedure SetTop(const Value: Boolean);
    procedure SetLeft(const Value: Boolean);
    procedure SetRight(const Value: Boolean);
    procedure SetBottom(const Value: Boolean);
  published
    property Top: Boolean Read FTop Write SetTop;
    property Left: Boolean Read FLeft Write SetLeft;
    property Right: Boolean Read FRight Write SetRight;
    property Bottom: Boolean Read FBottom Write SetBottom;
    property OnChange: TNotifyEvent Read FOnChange Write FOnChange;
  end;

  TReportFieldVar = record
    FAlignment: TAlignment;
    FCrossOut: TCrossOut;
    FontName: string[80];
    FontSize: Integer;
    FontStyle: TFontStyles;
    FontColor: TColor;
    BorderTop,
    BorderLeft,
    BorderRight,
    BorderBottom: Boolean;
    FBorderStyle: TPenStyle;
    FBorderWidth: Integer;
    FDividerHeight: TDividerHeight;
    FDividerStyle: TPenStyle;
    FMaxLength: Integer;
    FFixLength: Boolean;
    FColor: TColor;
    FWordWrap: Boolean;
    //
    Text: String[80];
    Hint: String[80];
    Transparent: Boolean;
    Top,
    Left,
    Width,
    Height: Integer;
  end;

  TReportField = class(TCustomLabel)
  private
    V: TReportFieldVar;
    FBorder: TBorder;
    FOnChange: TNotifyEvent;
    function GetText: TCaption;
    function GetTransparent: Boolean;
    procedure SetText(const Value: TCaption);
    procedure SetAlignment(Value: TAlignment);
    procedure SetBorder(const Value: TBorder);
    procedure SetBorderStyle(const Value: TPenStyle);
    procedure SetBorderWidth(const Value: Integer);
    procedure SetColor(const Value: TColor);
    procedure SetCrossOut(const Value: TCrossOut);
    procedure SetFixLength(const Value: Boolean);
    procedure SetMaxLength(const Value: Integer);
    procedure SetDividerHeight(const Value: TDividerHeight);
    procedure SetDividerStyle(const Value: TPenStyle);
    procedure SetTransparent(const Value: Boolean);
    procedure SetWordWrap(const Value: Boolean);
    procedure DoDrawText(var Rect: TRect; CText: String; Flags: Word);
    procedure BorderOnChange(Sender: TObject);
  protected
    procedure Paint; override;
  public
    function GetStyle: TReportFieldVar;
    procedure SetStyle(const Style: TReportFieldVar);
    constructor Create(AOwner: TComponent); override;
  published
    property Text: TCaption read GetText write SetText;
    property Border: TBorder Read FBorder Write SetBorder;
    property BorderStyle: TPenStyle Read V.FBorderStyle Write SetBorderStyle;
    property BorderWidth: Integer Read V.FBorderWidth Write SetBorderWidth;
    property CrossOut: TCrossOut Read V.FCrossOut Write SetCrossOut;
    property DividerHeight: TDividerHeight Read V.FDividerHeight
                                           Write SetDividerHeight;
    property DividerStyle: TPenStyle Read V.FDividerStyle Write SetDividerStyle;
    property MaxLength: Integer Read V.FMaxLength Write SetMaxLength;
    property FixLength: Boolean Read V.FFixLength Write SetFixLength;
    property Font;
    property Alignment: TAlignment Read V.FAlignment Write SetAlignment;
    property BackColor: TColor Read V.FColor Write SetColor;
    property Enabled;
    property ShowHint;
    property Transparent: Boolean Read GetTransparent Write SetTransparent;
    property Visible;
    property WordWrap: Boolean Read V.FWordWrap Write SetWordWrap;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStyleChange: TNotifyEvent Read FOnChange Write FOnChange;
  end;

procedure Register;

implementation

{*.DCR}

procedure Register;
begin
  RegisterComponents('Extra', [TReportField]);
end;

procedure TBorder.SetTop(const Value: Boolean);
begin
  FTop:=Value;
  if Assigned(OnChange) then OnChange(Self);
end;

procedure TBorder.SetLeft(const Value: Boolean);
begin
  FLeft:=Value;
  if Assigned(OnChange) then OnChange(Self);
end;

procedure TBorder.SetRight(const Value: Boolean);
begin
  FRight:=Value;
  if Assigned(OnChange) then OnChange(Self);
end;

procedure TBorder.SetBottom(const Value: Boolean);
begin
  FBottom:=Value;
  if Assigned(OnChange) then OnChange(Self);
end;

constructor TReportField.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with V do begin
    FColor:=Color;
    FBorderStyle:=psSolid;
    FBorderWidth:=1;
    FCrossOut:=coNone;
    FDividerHeight:=dhHalf;
    FDividerStyle:=psSolid;
    FMaxLength:=20;
    FFixLength:=True;
    FAlignment:=taLeftJustify;
    FBorder:=TBorder.Create;
    with FBorder do begin
      Top:=True;
      Left:=True;
      Right:=True;
      Bottom:=True;
    end;
  end;
  AutoSize:=False;
  Layout:=tlCenter;
  ShowAccelChar:=False;
  V.FWordWrap:=False;
  Border.OnChange:=BorderOnChange;
end;

procedure TReportField.BorderOnChange(Sender: TObject);
begin
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

function TReportField.GetText: TCaption;
begin
  Result:=Caption;
end;

procedure TReportField.SetText(const Value: TCaption);
begin
  if V.FFixLength and (Length(Value) > V.FMaxLength) then
    Caption:=copy(Value, 1, V.FMaxLength)
  else
    Caption:=Value;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetAlignment(Value: TAlignment);
begin
  if V.FAlignment <> Value then begin
    V.FAlignment:=Value;
    Paint;
  end;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

function TReportField.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TReportField.SetTransparent(const Value: Boolean);
begin
  if Transparent <> Value then begin
    if Value then
      ControlStyle:=ControlStyle - [csOpaque]
    else
      ControlStyle:=ControlStyle + [csOpaque];
    Paint;
  end;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetBorder(const Value: TBorder);
begin
  FBorder:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetBorderStyle(const Value: TPenStyle);
begin
  V.FBorderStyle:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetBorderWidth(const Value: Integer);
begin
  if Value < 0 then exit;
  V.FBorderWidth:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetColor(const Value: TColor);
begin
  V.FColor:=Value;
  Color:=V.FColor;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetCrossOut(const Value: TCrossOut);
begin
  V.FCrossOut:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetFixLength(const Value: Boolean);
begin
  if Value and (V.FMaxLength < 1) then begin
    if csDesigning in ComponentState then
      MessageDlg('Invalid Value for FixLength, MaxLength must be > 0!',
                 mtInformation, [mbOK], 0);
    exit;
  end;
  V.FFixLength:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetMaxLength(const Value: Integer);
begin
  if Value < 0 then exit;
  if V.FFixLength and (Value < 1) then begin
    if csDesigning in ComponentState then
      MessageDlg('Invalid Value for MaxLength, FixLength must be False!',
                 mtInformation, [mbOK], 0);
    exit;
  end;
  if Value < 0 then exit;
  V.FMaxLength:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetDividerHeight(const Value: TDividerHeight);
begin
  if not V.FFixLength then begin
    if csDesigning in ComponentState then
      MessageDlg('Invalid Value for DividerHeight, FixLength must be True!',
                 mtInformation, [mbOK], 0);
    exit;
  end;
  V.FDividerHeight:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetDividerStyle(const Value: TPenStyle);
begin
  if not V.FFixLength then begin
    if csDesigning in ComponentState then
      MessageDlg('Invalid Value for DividerStyle, FixLength must be True!',
                 mtInformation, [mbOK], 0);
    exit;
  end;
  V.FDividerStyle:=Value;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.SetWordWrap(const Value: Boolean);
begin
  if V.FWordWrap <> Value then begin
    V.FWordWrap := Value;
    Paint;
  end;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

procedure TReportField.DoDrawText(var Rect: TRect; CText: String; Flags: Word);
begin
  if (Flags and DT_CALCRECT <> 0) and ((CText = '') or ShowAccelChar and
     (CText[1] = '&') and (CText[2] = #0)) then CText := CText + ' ';
  if not ShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Canvas.Font:=Font;
  if not Enabled then
    begin
      OffsetRect(Rect, 1, 1);
      Canvas.Font.Color:=clBtnHighlight;
      DrawText(Canvas.Handle, PChar(CText), Length(CText), Rect, Flags);
      OffsetRect(Rect, -1, -1);
      Canvas.Font.Color:=clBtnShadow;
      DrawText(Canvas.Handle, PChar(CText), Length(CText), Rect, Flags);
    end
  else
    DrawText(Canvas.Handle, PChar(CText), Length(CText), Rect, Flags);
end;

procedure TReportField.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect, CalcRect: TRect;
  DrawStyle: Integer;
  I, c1, c2, cw: Integer;
begin
  with Canvas do begin
    if not Transparent then begin
      Brush.Color:=Self.Color;
      Brush.Style:=bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style:=bsClear;
    // now here we draw the lines
    // Border
    Pen.Style:=V.FBorderStyle;
    Pen.Width:=1;
    For I:=0 to V.FBorderWidth - 1 do begin
      MoveTo(I + Pen.Width div 2, Pen.Width div 2);
      if FBorder.Left then
        LineTo(I + Pen.Width div 2, ClientHeight - Pen.Width + Pen.Width div 2);
      //
      MoveTo(Pen.Width div 2, (ClientHeight - Pen.Width + Pen.Width div 2) - I);
      if FBorder.Bottom then
        LineTo(ClientWidth - Pen.Width + Pen.Width div 2,
               (ClientHeight - Pen.Width + Pen.Width div 2) - I);
      //
      MoveTo((ClientWidth - Pen.Width + Pen.Width div 2) - I,
             ClientHeight - Pen.Width + Pen.Width div 2);
      if FBorder.Right then
        LineTo((ClientWidth - Pen.Width + Pen.Width div 2) - I, Pen.Width div 2);
      //
      MoveTo(ClientWidth - Pen.Width + Pen.Width div 2, (Pen.Width div 2) + I);
      if FBorder.Top then
        LineTo(Pen.Width div 2, (Pen.Width div 2) + I);
    end;
    // Divider
    if V.FFixLength and (V.FDividerStyle <> psClear) then begin
      for I:=1 to MaxLength - 1 do begin
        cw:=Round(((ClientWidth - (2 * V.FBorderWidth)) / V.FMaxLength) * I)
                  + V.FBorderWidth;
        MoveTo(cw, ClientHeight - Pen.Width + 1 + Pen.Width div 2);
        Pen.Style:=V.FDividerStyle;
        Pen.Width:=1;
        case V.FDividerHeight of
          dhFull: LineTo(cw, Pen.Width div 2);
          dhHalf: LineTo(cw, ClientHeight div 2);
        end;
      end;
    end;
    // CrossOut
    if V.FCrossOut In [coSimple, coDouble] then begin
      Pen.Style:=psSolid;
      Pen.Width:=1;
      MoveTo(0, ClientHeight);
      LineTo(ClientWidth, 0);
      if V.FCrossOut = coDouble then begin
        MoveTo(0, 0);
        LineTo(ClientWidth, ClientHeight);
      end;
    end;
    // Text
    if V.FFixLength and (Length(GetText) > 0) then
      begin
        cw:=Length(GetText);
        case V.FAlignment of
          taCenter: begin c1:=1 + (V.FMaxLength - cw) div 2; c2:=c1 + cw - 1; end;
          taRightJustify: begin c2:=V.FMaxLength; c1:=c2 - cw + 1; end;
        else
          begin c1:=1; c2:=cw; end;
        end;
        DrawStyle:=Alignments[taCenter];
        for I:=c1 to c2 do begin
          Rect.Left:=Round(((ClientWidth - (2 * V.FBorderWidth)) / V.FMaxLength)
                           * (I - 1)) + V.FBorderWidth + 1;
          Rect.Top:=V.FBorderWidth;
          Rect.Bottom:=ClientHeight - V.FBorderWidth;
          Rect.Right:=Round(((ClientWidth - (2 * V.FBorderWidth)) / V.FMaxLength) * I)
                            + V.FBorderWidth;
          CalcRect:=Rect;
          DoDrawText(CalcRect, GetText[I - c1 + 1], DrawStyle or DT_CALCRECT);
          OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
          DoDrawText(Rect, GetText[I - c1 + 1], DrawStyle);
        end;
      end
    else
      begin
        Rect:=ClientRect;
        InflateRect(Rect, -V.FBorderWidth, -V.FBorderWidth);
        DrawStyle:=DT_EXPANDTABS or WordWraps[V.FWordWrap] or Alignments[V.FAlignment];
        CalcRect:=Rect;
        DoDrawText(CalcRect, GetText, DrawStyle or DT_CALCRECT);
        OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
        DoDrawText(Rect, GetText, DrawStyle);
      end;
  end;
end;

function TReportField.GetStyle: TReportFieldVar;
begin
  V.Text:=GetText;
  V.Hint:=Hint;
  V.Transparent:=Transparent;
  V.BorderTop:=FBorder.Top;
  V.BorderLeft:=FBorder.Left;
  V.BorderRight:=FBorder.Right;
  V.BorderBottom:=FBorder.Bottom;
  V.FontName:=Font.Name;
  V.FontSize:=Font.Size;
  V.FontStyle:=Font.Style;
  V.FontColor:=Font.Color;
  V.Top:=Top;
  V.Left:=Left;
  V.Width:=Width;
  V.Height:=Height;
  Result:=V;
end;

procedure TReportField.SetStyle(const Style: TReportFieldVar);
begin
  V:=Style;
  SetText(Style.Text);
  Hint:=V.Hint;
  Transparent:=V.Transparent;
  FBorder.Top:=V.BorderTop;
  FBorder.Left:=V.BorderLeft;
  FBorder.Right:=V.BorderRight;
  FBorder.Bottom:=V.BorderBottom;
  Font.Name:=V.FontName;
  Font.Size:=V.FontSize;
  Font.Style:=V.FontStyle;
  Font.Color:=V.FontColor;
  Top:=V.Top;
  Left:=V.Left;
  Width:=V.Width;
  Height:=V.Height;
  Color:=V.FColor;
  Paint;
  if Assigned(OnStyleChange) then OnStyleChange(Self);
end;

end.

